@@ -13,19 +13,23 @@
 #include <QStorageInfo>
 
 CachedNetworkAccessManager::CachedNetworkAccessManager(const QString &path, QObject *parent)
-    : KIO::AccessManager(parent)
+    : QNetworkAccessManager(parent)
 {
     const QString cacheDir = QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QLatin1Char('/') + path;
     QNetworkDiskCache *cache = new QNetworkDiskCache(this);
-    QStorageInfo storageInfo(cacheDir);
+    QStorageInfo storageInfo(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
     cache->setCacheDirectory(cacheDir);
     cache->setMaximumCacheSize(storageInfo.bytesTotal() / 1000);
     setCache(cache);
+
+    setTransferTimeout();
 }
 
 QNetworkReply *CachedNetworkAccessManager::createRequest(Operation op, const QNetworkRequest &request, QIODevice *outgoingData)
 {
     QNetworkRequest req(request);
     req.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::PreferCache);
-    return KIO::AccessManager::createRequest(op, request, outgoingData);
+    return QNetworkAccessManager::createRequest(op, request, outgoingData);
 }
+
+#include "moc_CachedNetworkAccessManager.cpp"
