@@ -6,22 +6,22 @@
  */
 
 #include "FlatpakRefreshAppstreamMetadataJob.h"
-#include <QDebug>
+#include "libdiscover_backend_flatpak_debug.h"
 
 FlatpakRefreshAppstreamMetadataJob::FlatpakRefreshAppstreamMetadataJob(FlatpakInstallation *installation, FlatpakRemote *remote)
     : QThread()
     , m_cancellable(g_cancellable_new())
-    , m_installation(installation)
-    , m_remote(remote)
+    , m_installation(g_object_ref(installation))
+    , m_remote(g_object_ref(remote))
 {
-    g_object_ref(m_remote);
     connect(this, &FlatpakRefreshAppstreamMetadataJob::finished, this, &QObject::deleteLater);
 }
 
 FlatpakRefreshAppstreamMetadataJob::~FlatpakRefreshAppstreamMetadataJob()
 {
-    g_object_unref(m_remote);
     g_object_unref(m_cancellable);
+    g_object_unref(m_installation);
+    g_object_unref(m_remote);
 }
 
 void FlatpakRefreshAppstreamMetadataJob::cancel()
@@ -48,7 +48,10 @@ void FlatpakRefreshAppstreamMetadataJob::run()
     if (!flatpak_installation_update_appstream_sync(m_installation, flatpak_remote_get_name(m_remote), nullptr, nullptr, m_cancellable, &localError)) {
 #endif
         const QString error = localError ? QString::fromUtf8(localError->message) : QStringLiteral("<no error>");
-        qWarning() << "Failed to refresh appstream metadata for " << flatpak_remote_get_name(m_remote) << ": " << error;
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG).nospace()
+            << "Failed to refresh appstream metadata for " << flatpak_remote_get_name(m_remote) << ": " << error;
     }
     Q_EMIT jobRefreshAppstreamMetadataFinished(m_installation, m_remote);
 }
+
+#include "moc_FlatpakRefreshAppstreamMetadataJob.cpp"
