@@ -28,7 +28,7 @@ public:
     QString comment() override;
     QString name() const override;
     QString packageName() const override;
-    QStringList categories() override;
+    bool hasCategory(const QString &category) const override;
     QUrl homepage() override;
     QJsonArray licenses() override;
     QString longDescription() override;
